#include <iostream>
#include <memory>

template <typename T>
class MyContainer {
	size_t size_;
	std::unique_ptr<T[]> array_;

	template <typename U>
	class Iterator {
		// satisfies the ForwardIterator concept
		U* ptr;

	public:
		using value_type = T;
		using difference_type = std::ptrdiff_t;
		using pointer = U*;
		using reference = U&;
		using iterator_category = std::forward_iterator_tag;

		explicit Iterator(U* ptr)
		    : ptr(ptr) {}
		friend bool operator==(const Iterator& a, const Iterator& b) {
			return a.ptr == b.ptr;
		}
		friend bool operator!=(const Iterator& a, const Iterator& b) {
			return !(a == b);
		}
		reference operator*() { return *ptr; }
		pointer operator->() { return ptr; }
		Iterator& operator++() {
			++ptr;
			return *this;
		}
		Iterator operator++(int) {
			Iterator result = *this;
			++*this;
			return result;
		}
	};

public:
	using iterator = Iterator<T>;
	using const_iterator = Iterator<const T>;

	explicit MyContainer(size_t s)
	    : size_(s)
	    , array_(std::make_unique<T[]>(size_)) {}

	size_t size() const { return size_; }
	const T& operator[](size_t index) const { return array_[index]; }
	T& operator[](size_t index) { return array_[index]; }

	iterator begin() { return iterator(array_.get()); }
	iterator end() { return iterator(array_.get() + size_); }
	const_iterator begin() const { return const_iterator(array_.get()); }
	const_iterator end() const { return const_iterator(array_.get() + size_); }
};

int main() {
	MyContainer<double> cont(10);
	for (size_t i = 0; i < cont.size(); ++i) {
		cont[i] = i;
	}
	for (double d : cont) {
		std::cout << d << ' ';
	}
	const auto& cref = cont;
	for (double d : cref) {
		std::cout << d << ' ';
	}
}
