#include <iostream>
#include <memory>

template <typename T>
class MyContainer {
	size_t size_;
	std::unique_ptr<T[]> array_;

public:
	explicit MyContainer(size_t s)
	    : size_(s)
	    , array_(std::make_unique<T[]>(size_)) {}

	size_t size() const { return size_; }

#if 1 // change this to 0 to switch to the C++14 version with auto
	const T& operator[](size_t index) const {
		return _get<const T>(*this, index);
	}

	T& operator[](size_t index) {
		return _get<T>(*this, index);
	}

private:
	template <typename RetVal, typename Self>
	static RetVal& _get(Self& self, size_t index) {
		return self.array_[index];
	}
#else
	const T& operator[](size_t index) const {
		return _get(*this, index);
	}

	T& operator[](size_t index) {
		return _get(*this, index);
	}

private:
	template <typename Self>
	static auto& _get(Self& self, size_t index) {
		return self.array_[index];
	}
#endif
};

int main() {
	MyContainer<double> cont(10);
	const auto& ref = cont;
	for (size_t i = 0; i < cont.size(); ++i) {
		cont[i] = i;
	}
	for (size_t i = 0; i < cont.size(); ++i) {
		std::cout << ref[i] << ' ';
	}
}
